module Language.Haskell.ER.Syntax 
       ( Code(..)
       , showName
--       , quote
--       , quoteCode
       , mkLamE 
       , mkLetE 
       ) where

import Language.Haskell.TH
import Language.Haskell.TH.Syntax
import Data.PackedString
import Control.Monad

import GHC.Base	-- for I#

-- Our basic type for rewriting, a piece of code
-- and other missing parts of the TH module.

data Code = Code [Dec]
     deriving (Eq,Show)

instance Lift Code where
#ifdef __HADDOCK__
  {}
#else  
  lift (Code decs) = [| Code decs |]
#endif

-- Utils

mkLamE [] exp   = exp
mkLamE pats exp = LamE pats exp

mkLetE []   e = e
mkLetE decs e = LetE decs e


-- Some global instances

instance Show NameFlavour where
#ifdef __HADDOCK__
  {}
#else  
   show NameS = "NameS"
   show (NameQ n) = "NameQ " ++ show n
   show (NameU i) = "NameU " ++ show (I# i)
   show (NameL i) = "NameL " ++ show (I# i)
#if __GLASGOW_HASKELL__ < 605
   show (NameG ns n) = "NameG " ++ show ns ++ " " ++ show n
#else
   show (NameG ns n x) = "NameG " ++ show ns ++ " " ++ show n ++ " " ++ show x
#endif
#endif

instance Show NameSpace where
   show VarName = "VarName"
   show DataName = "DataName"
   show TcClsName = "TcClsName"

--showName :: Name -> String
--showName (Name nm flvr) = show nm ++ " " ++ show flvr


instance Lift Dec where 
#ifdef __HADDOCK__
  {}
#else
  lift (FunD nm cs) = [| FunD nm cs |]
  lift (ValD p b ds) = [| ValD p b ds |]
  lift (DataD cxt n ns1 cs ns2) = [| DataD cxt n ns1 cs ns2 |]
  lift (NewtypeD cxt nm ns1 c ns2) = [| NewtypeD cxt nm ns1 c ns2 |]
  lift (TySynD n ns ty) = [| TySynD n ns ty |]
  lift (ClassD cxt n nm fd ds) = [| ClassD cxt n nm fd ds |]
  lift (InstanceD cxt ty ds) = [| InstanceD cxt ty ds |]
  lift (SigD nm ty) = [| SigD nm ty |]
  lift (ForeignD f ) = [| ForeignD f  |]
#endif

instance Lift Exp where
#ifdef __HADDOCK__
  {}
#else
  lift (VarE nm) = [| VarE nm |]
  lift (ConE nm) = [| ConE nm |]
  lift (LitE lit) = [| LitE lit |]
  lift (AppE e1 e2) = [| AppE e1 e2 |]
  lift (InfixE me1 e2 me3) = [| InfixE me1 e2 me3 |]
  lift (LamE ps e ) = [| LamE ps e  |]
  lift (TupE es) = [| TupE es |]
  lift (CondE e1 e2 e3) = [| CondE e1 e2 e3 |]
  lift (LetE decs e) = [| LetE decs e |]
  lift (CaseE e ms) = [| CaseE e ms |]
  lift (DoE stmts) = [| DoE stmts |]
  lift (CompE stmts) = [| CompE stmts |]
  lift (ArithSeqE range) = [| ArithSeqE range |]
  lift (ListE es) = [| ListE es |]
  lift (SigE e t) = [| SigE e t |]
  lift (RecConE nm fs) = [| RecConE nm fs |]
  lift (RecUpdE e fs) = [| RecUpdE e fs |]
#endif

instance Lift Lit where
#ifdef __HADDOCK__
  {}
#else
  lift (CharL ch) = [| CharL ch |]
  lift (StringL str) = [| StringL str |]
  lift (IntegerL i) = [| IntegerL i |]
  lift (RationalL r) = [| RationalL r |]
  lift (IntPrimL i) = [| IntPrimL i |]
  lift (FloatPrimL r) = [| FloatPrimL r |]
  lift (DoublePrimL r) = [| DoublePrimL r |]
#endif

instance Lift Rational where lift _ = error "Rational.. what are you doing!"

instance Lift Pat where
#ifdef __HADDOCK__
  {}
#else
  lift (LitP lit) = [| LitP lit |]
  lift (VarP nm) = [| VarP nm |]
  lift (TupP ps) = [| TupP ps |]
  lift (ConP nm ps) = [| ConP nm ps |]
  lift (InfixP p1 nm p2) = [| InfixP p1 nm p2 |]
  lift (TildeP p) = [| TildeP p |]
  lift (AsP nm p) = [| AsP nm p |]
  lift (WildP) = [| WildP |]
  lift (RecP nm fs) = [| RecP nm fs |]
  lift (ListP ps) = [| ListP ps |]
  lift (SigP p ty) = [| SigP p ty |]
#endif
  
instance Lift Body where
#ifdef __HADDOCK__
  {}
#else
  lift (GuardedB ges) = [| GuardedB ges |]
  lift (NormalB e) = [| NormalB e |]
#endif

instance Lift Con where
#ifdef __HADDOCK__
  {}
#else
  lift (NormalC nm sts) = [| NormalC nm sts |]
  lift (RecC nm vsts) = [| RecC nm vsts |]
  lift (InfixC st1 nm st2) = [| InfixC st1 nm st2 |]
  lift (ForallC ns cxt con) = [| ForallC ns cxt con |]
#endif

instance Lift Clause where
#ifdef __HADDOCK__
  {}
#else
  lift (Clause ps b ds) = [| Clause ps b ds |]
#endif

instance Lift Guard where 
#ifdef __HADDOCK__
  {}
#else
  lift (NormalG e) = [| NormalG e |]
  lift (PatG stmts) = [| PatG stmts |]
#endif

instance Lift Strict where 
#ifdef __HADDOCK__
  {}
#else
  lift (IsStrict ) = [| IsStrict  |]
  lift (NotStrict) = [| NotStrict |]
#endif

instance Lift FunDep where 
#ifdef __HADDOCK__
  {}
#else
  lift (FunDep ns1 ns2) = [| FunDep ns1 ns2 |]
#endif

instance Lift Foreign where
#ifdef __HADDOCK__
  {}
#else
  lift (ImportF conv safety str nm ty) = [| ImportF conv safety str nm ty |]
  lift (ExportF conv str nm ty) = [| ExportF conv str nm ty |]
#endif

instance Lift Callconv where
#ifdef __HADDOCK__
  {}
#else
  lift (CCall) = [| CCall |]
  lift (StdCall) = [| StdCall |]
#endif

instance Lift Safety where
#ifdef __HADDOCK__
  {}
#else
  lift (Unsafe ) = [| Unsafe  |]
  lift (Safe ) = [| Safe  |]
  lift (Threadsafe) = [| Threadsafe |]
#endif

instance Lift Match where
#ifdef __HADDOCK__
  {}
#else
  lift (Match p b ds) = [| Match p b ds |]
#endif

instance Lift Stmt where
#ifdef __HADDOCK__
  {}
#else
  lift (BindS p e) = [| BindS p e |]
  lift (LetS ds) = [| LetS ds |]
  lift (NoBindS e) = [| NoBindS e |]
  lift (ParS stmtss) = [| ParS stmtss |]
#endif

instance Lift Range where
#ifdef __HADDOCK__
  {}
#else
  lift (FromR e1) = [| FromR e1 |]
  lift (FromThenR e1 e2) = [| FromThenR e1 e2 |]
  lift (FromToR e1 e2) = [| FromToR e1 e2 |]
  lift (FromThenToR e1 e2 e3) = [| FromThenToR e1 e2 e3 |]
#endif
 
instance Lift Type where
#ifdef __HADDOCK__
  {}
#else
  lift (ForallT ns cxt ty) = [| ForallT ns cxt ty |]
  lift (VarT nm) = [| VarT nm |]
  lift (ConT nm) = [| ConT nm |]
  lift (TupleT i) = [| TupleT i |]
  lift (ArrowT) = [| ArrowT |]
  lift (ListT) = [| ListT |]
  lift (AppT ty1 ty2) = [| AppT ty1 ty2 |]
#endif

#if __GLASGOW_HASKELL__ < 605
instance (Lift a) => Lift (Maybe a) where
#ifdef __HADDOCK__
  {}
#else
  lift (Nothing ) = [| Nothing  |]
  lift (Just a) = [| Just a |]
#endif
#endif

instance Lift Name where
#ifdef __HADDOCK__
  {}
#else
  lift (Name name flavor) = [| Name name flavor |]
#endif

instance Lift PackedString where 
#ifdef __HADDOCK__
  {}
#else
  lift ps = [| packString txt |]
    where txt = unpackPS ps
#endif

instance Lift NameFlavour where
#ifdef __HADDOCK__
  {}
#else
  lift NameS =     [| NameS |]
  lift (NameQ m) = [| NameQ m |]
  lift (NameU i#) = [| myNameU i |] where i = I# i#
  lift (NameL i#) = [| myNameL i |] where i = I# i#
#if __GLASGOW_HASKELL__ < 605
  lift (NameG ns m) = [| NameG ns m |]
#else
  lift (NameG ns m x) = [| NameG ns m x |]
#endif
#endif



instance Lift NameSpace where
#ifdef __HADDOCK__
  {}
#else
  lift VarName = [| VarName |]
  lift DataName = [| DataName |]
  lift TcClsName = [| TcClsName |]
#endif

myNameU (I# i) = NameU i
myNameL (I# i) = NameL i
